//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Bivariate2DInterpolantScattered.cc --- Implementation of
//											Bivariate2DInterpolantScattered
//
//	DESCRIPTION:
//		Implementation of the Bivariate2DInterpolantScattered class via the
//		TOMS routine QSHEP2. cfortran.h is used to interface to TOMS.
//
//	CHANGE HISTORY:
//		UL, 2000-01-11: 1.0.0: - first version
//		UL, 2000-01-25: 1.1.0: - changed from TOMS526 to TOMS660
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/slatec/cxx/Bivariate2DInterpolantScattered.cc,v 1.1 2000/01/25 17:55:25 ulammers Exp $
//

#include "slatec++.h"
#include "slatec.h"
#include "errstr.h"
#include "Tracer.h"

#include "Bivariate2DInterpolantScattered.h"

namespace Slatec {

Bivariate2DInterpolantScattered<float>::Bivariate2DInterpolantScattered(
											const vector<float> &x,
											const vector<float> &y,
											const vector<float> &z)
{
	PROC

	if (x.size() != y.size() || x.size() != z.size())
		errstr<<"Vectors with data points to define bivariate interpolator "
				"have different sizes"<<Sas::error("VectorSizeMismatch");

	if (x.size() < 6)
		errstr<<"Need at least six data points to define bi-variate "
				"interpolator"<<Sas::error("TooLittleDataPoints");

	_n		= x.size();
	_xd		= new float[_n];
	_yd		= new float[_n];
	_f		= new float[_n];
	_nq		= 13;	//	value recommended in TOMS660 description
	_nw		= 19;	//	value recommended in TOMS660 description
	_nr		= INT(sqrt(DBL(_n)/3.))+1;
					//	value recommended in TOMS660 description
	_lcell	= new int [_nr*_nr];
	_lnext	= new int [_n];
	_rsq	= new float [_n];
	_a		= new float [5*_n];

	for (int n=0; n<_n; ++n) {
		_xd[n] = x[n];
		_yd[n] = y[n];
		_f[n] = z[n];
	}

	int ierr = 0;
	TOMS_QSHEP2(_n, _xd, _yd, _f, _nq, _nw, _nr, _lcell, _lnext,
				_xmin, _ymin, _dx, _dy, _rmax, _rsq, _a, ierr);

	if (ierr) {
		errstr<<"In setting up 2D-interpolation routine (TOMS660) ";
		switch (ierr) {
			case 1:	errstr<<"N, NQ, NW, or NR is out of range.";
					break;
			case 2: errstr<<"duplicate nodes were encountered.";
					break;
			case 3: errstr<<"all nodes are collinear.";
					break;
			default:
					errstr<<"an unknown error "<<ierr<<" occured.";
					break;
		}
		errstr<<Sas::error("NumericalError");
	}

	ENDPROC
} // Bivariate2DInterpolantScattered::Bivariate2DInterpolantScattered()


void Bivariate2DInterpolantScattered<float>::additionalDataPoints(unsigned)
{
	PROC

	errstr<<"This is not implemented with currently used numerical "
			"2D-interpolation scheme (TOMS660)"
		  <<Sas::warning("CallIgnored");

	ENDPROC
} // Bivariate2DInterpolantScattered<float>::additionalDataPoints()


float Bivariate2DInterpolantScattered<float>::operator()(float x, float y) const
{
	PROC

	float z;
	TOMS_WRAP_QS2VAL(x, y, _n, _xd, _yd, _f, _nr, _lcell, _lnext,
				     _xmin, _ymin, _dx, _dy, _rmax, _rsq, _a, z);

	RETURN z;
} // Bivariate2DInterpolantScattered<float>::operator()


Bivariate2DInterpolantScattered<float>::~Bivariate2DInterpolantScattered()
{
	PROC

	delete [] _xd; _xd = 0;
	delete [] _yd; _yd = 0;
	delete [] _f; _f = 0;
	delete [] _lcell; _lcell = 0;
	delete [] _lnext; _lnext = 0;
	delete [] _rsq; _rsq = 0;
	delete [] _a; _a = 0;

	ENDPROC
} // Bivariate2DInterpolantScattered<double>::~Bivariate2DInterpolantScattered()

}

