/* written by John Doug Reynolds, January 2000 */

#include "Convolver.h"
#include <slatec.h>
#include <errstr.h>

Convolver::Convolver( unsigned size )
   : _length(0)
{
   resize( size );
}

void	Convolver::resize( unsigned size )
{
   if ( size != _length ) {
      _length = size;

      _signal.resize( _length );
      _WA.resize( _length );
      _CH.resize( _length );

      SLATEC_RFFTI1( _length, _WA.begin(), _IFAC );
   }
   reset();
}

void	Convolver::reset( enum Type which )
{
   if ( which & SIGNAL ) _isOpen = false;
   if ( which & KERNEL ) _kernel.clear();
}

void	Convolver::testSize( const vector<float> &v ) const
{
   if ( v.size() != _length )
      errstr << "Attempt to load vector size " << v.size()
	     << " after initialization to size " << _length
	     << Sas::error("Convolver:sizeError");
}

void	Convolver::load( const vector<float> &v, enum Type to )
{
   testSize(v);
   load( v.begin(), to );
}

void	Convolver::load( vector<float> &v, enum Type to )
{
   testSize(v);
   load( v.begin(), to );
}

void Convolver::loadAsConst( vector<float> &v, enum Type to )
{
   testSize(v);
   load( static_cast<const float*>(v.begin()), to );
}

void Convolver::loadAsConst( float *v, enum Type to )
{
   load( static_cast<const float*>(v), to );
}

void	Convolver::combine( float *dst, const float *src )
{
   if ( _length == 0 ) return;

   unsigned count = (_length - 1) / 2;

   *dst++ *= *src++;

   while ( count-- ) {
      float srcR = *src++;
      float srcI = *src++;

      float dstR = dst[0];
      float dstI = dst[1];

      *dst++ = dstR * srcR - dstI * srcI;
      *dst++ = dstR * srcI + dstI * srcR;
   }

   if ( (_length & 1) == 0 ) *dst *= *src;
}

void	Convolver::load( const float *v, enum Type to )
{
   vector<float> temp( v, v + _length );
   load( temp.begin(), to );
}

void	Convolver::load( float *src, enum Type to )
{
   SLATEC_RFFTF1( _length, src, _CH.begin(), _WA.begin(), _IFAC );

   switch ( to ) {
   case KERNEL:
      if ( _kernel.empty() ) {
	 _kernel.resize( _length );
	 float *dst = _kernel.begin();
	 for ( unsigned count = _length; count--; *dst++ = *src++ );
      }
      else combine( _kernel.begin(), src );
      break;

   case SIGNAL:
      if ( ! _isOpen ) {
	 float *dst = _signal.begin(), norm = 1.0 / _length;
	 for ( unsigned count = _length; count--; *dst++ = *src++ * norm );
	 _isOpen = true;
      }
      else combine( _signal.begin(), src );
      break;

   default:
      errstr << "load(," << int(to) << "): target must be "
	     << int(SIGNAL) << " (SIGNAL) or " << int(KERNEL) << " (KERNEL)"
	     << Sas::error("Convolver:invalid");
   }
}

const vector<float>&	Convolver::operator() (void)
{
   if ( _isOpen ) {
      if ( ! _kernel.empty() ) combine( _signal.begin(), _kernel.begin() );
      SLATEC_RFFTB1(_length,_signal.begin(),_CH.begin(),_WA.begin(),_IFAC);
      _isOpen = false;
   }
   return _signal;
}
