/* written by John Doug Reynolds, January 2000 */

#ifndef CONVOLVER_H
#define CONVOLVER_H

#include <vector>

/*

  This class convolves an arbitrary number of curves simultaneously.

  Curves may be loaded as SIGNAL or KERNEL: once defined, the kernel
  is automatically convolved along with the accumulated signals until
  disabled by resize() or reset(KERNEL).  Any number of curves may be
  loaded as SIGNAL or KERNEL before convolving.  The purpose of the
  kernel is simply to avoid wasteful recomputations of the FFT when
  the same response curve will be applied to a sequence of signal
  curves.  There is no need to bother with the kernel except in that
  situation.

  Initialize a class instance to a given dimension using either the
  constructor or resize().  Each curve passed to load() is required to
  have that same dimension.  loadAsConst() treats its non-const input
  as if it were const; otherwise the FFT is performed in-place and the
  input curve is replaced with its transform.  The first call to
  operator() after load() performs and returns the convolution.
  Subsequent calls return the previous result without doing any work.
  The storage for the previous result remains valid until the next
  load() or resize().

  reset(SIGNAL) aborts the accumulation of SIGNAL curves without
  performing the convolution, leaving the previous result undefined.

 */

class Convolver {
public:
   enum Type { SIGNAL = 1, KERNEL = 2 };

   unsigned size() const { return _length; }

   Convolver( unsigned size = 0 );

   void resize( unsigned size );
   void reset( enum Type which = Type(SIGNAL + KERNEL) );

   void load( float*,	      enum Type to = SIGNAL );
   void load( vector<float>&, enum Type to = SIGNAL );

   void load( const float*,	    enum Type to = SIGNAL );
   void load( const vector<float>&, enum Type to = SIGNAL );

   void loadAsConst( float*,	     enum Type to = SIGNAL );
   void loadAsConst( vector<float>&, enum Type to = SIGNAL );

   const vector<float>& operator() (void);

private:
   unsigned			_length;	// initialized array length
   vector<float>		_kernel;	// reusable response function
   vector<float>		_signal;	// accumulator and result
   bool				_isOpen;	// result not yet computed

   void testSize( const vector<float> &v ) const;
   void combine( float *dst, const float *src );

   // SLATEC private work space

   int				_IFAC[15];
   vector<float>		_WA, _CH;
};

#endif /* CONVOLVER_H */
