//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	LinearInterpolant.h --- Implementation of Interpolant class as piecewise
//							linear functions
//
//	DESCRIPTION:
//		A LinearInterpolant object represents a monotone piecewise
//		linear interpolant to given data.
//
//	CHANGE HISTORY:
//		UL, 2000-09-28: 1.0.0: - first version
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/slatec/cxx/LinearInterpolant.h,v 1.4 2001/11/22 10:07:07 ulammers Exp $
//

#ifndef LINEAR_INTERPOLANT_H
#define LINEAR_INTERPOLANT_H

#include <vector>
#include "Interpolant.h"


namespace Slatec {

template<class NumericType>
class LinearInterpolant : public virtual Interpolant<NumericType> {
public:
	LinearInterpolant();
	LinearInterpolant(NumericType a);
	LinearInterpolant(const vector<NumericType> &x,
					  const vector<NumericType> &y);
		//
		//	PURPOSE:
		//		Instantiate a LinearInterpolant object
		//
		//	ARGUMENTS:
		//		x	: vector of independent variable values
		//		y	: vector of dependent variable values corresponding to <x>
		//


	LinearInterpolant(const LinearInterpolant<NumericType> &);
	LinearInterpolant<NumericType> &operator=(
									const LinearInterpolant<NumericType> &);
		//
		//	PURPOSE:
		//		(Deep) copy construction/assignment
		//


	NumericType operator()(NumericType x) const;
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <X>
		//


	LinearInterpolant<NumericType> &operator*=(NumericType);
	LinearInterpolant<NumericType> &operator*=
									(const Interpolant<NumericType> &);
	LinearInterpolant<NumericType> &operator*=
									(const vector<NumericType> &);
	LinearInterpolant<NumericType> &operator/=
									(const Interpolant<NumericType> &);
	LinearInterpolant<NumericType> &operator+=
									(const Interpolant<NumericType> &);
	LinearInterpolant<NumericType> &operator+=(NumericType);
		//
		//	PURPOSE:
		//		Perform arithmetic on LinearInterpolant
		//


	NumericType integral(NumericType x1, NumericType x2) const;
		//
		//	PURPOSE:
		//		Calculate integral of interpolant between <x1> and <x2>
		//


	Interpolant<NumericType> &supplement(
									const std::vector<NumericType> &,
									const std::vector<NumericType> &);
		//
		//	PURPOSE:
		//		Supplement object by additional data points
		//
		//	NOTE: Returning "LinearInterpolant<NumericType> &" here gives
		//		LinearInterpolant.h:115: sorry, not implemented:
		//			adjusting pointers for covariant returns
		//	with gcc-2.95.2 - to be revisited with gcc-3
		//
  

	~LinearInterpolant();
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//


	void printOn(ostream &) const;

private:
	unsigned					_n;
	mutable unsigned			_startLi;
	std::vector<NumericType>	_y;
	NumericType					_f, _a;

	void validate() const;
	inline double y(unsigned n) const { return _f * _y[n] + _a; }
	unsigned leftIndex(NumericType x, unsigned startAt = 0) const;
};


template<class NumericType>
LinearInterpolant<NumericType> &operator*(
								LinearInterpolant<NumericType> &f1,
								const LinearInterpolant<NumericType> &f2)
{
	PROC
	RETURN f1 *= f2;
} // operator*()

template<class NumericType>
LinearInterpolant<NumericType> &operator+(
								LinearInterpolant<NumericType> &f1,
								const LinearInterpolant<NumericType> &f2)
{
	PROC
	RETURN f1 += f2;
} // operator*()

template<class NumericType>
LinearInterpolant<NumericType> &operator+(
								LinearInterpolant<NumericType> &f1,
								NumericType a)
{
	PROC
	RETURN f1 += a;
} // operator*()

template<class NumericType>
LinearInterpolant<NumericType> &operator-(
								NumericType a,
								LinearInterpolant<NumericType> &f1)
{
	PROC

	f1 *= -1.;
	RETURN f1 += a;
} // operator*()

template<class NumericType>
LinearInterpolant<NumericType> &operator*(
								LinearInterpolant<NumericType> &f1,
								NumericType f)
{
	PROC
	RETURN f1 *= f;
} // operator*()


}

template<class NumericType>
ostream &operator<<(ostream &,
					const Slatec::LinearInterpolant<NumericType> &);

#endif
